/*
Copyright (c) 2009, Dan "Ducky" Little & GeoMOOSE.org

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

function GeomooseCatalog(parentDivId, olMap) {
	var elementCounter = 0;
	var MyCatalog = false;
	var MySelf = this;

	var MapbookLoaded = false;

	function getCatalogElementId() {
		elementCounter++;
		return 'catalog-'+elementCounter;
	}

	function toggleGroup() {
		/*var g = document.getElementById(groupElementId);*/
		var grpList = this.nextSibling;
		var className = new String(grpList.className);
		if(className.match('catalog-group-collapsed')) {
			grpList.className = className.replace('collapsed','expanded');
		} else {
			grpList.className = className.replace('expanded','collapsed');
		}
	}


	function groupClickAll(event) {
		if(window.event) { event =window.event; }
		event.cancelBubble = true;

		var group = this.parentNode.nextSibling;	
		var layers = group.getElementsByTagName('input');

		for(var i = 0; i < layers.length; i++) {
			if(layers[i].parentNode.className == 'catalog-layer') {
				if(layers[i].checked != this.checked) {
					MySelf.setLayerVisibility(layers[i].value, this.checked);
					layers[i].checked = this.checked;
				}
			}
		}
	}

	function renderGroup(groupElementId, group) {
		var p = document.getElementById(groupElementId);
		var ul = document.createElement('ul');
		ul.className = 'catalog';

		if(p.tagName == 'UL' && group.getAttribute('title')) {
			var li = document.createElement('li');
			p.appendChild(li);

			if(CONFIGURATION.group_checkboxes && parseBoolean(group.getAttribute('multiple'), true)) {
				var group_control = document.createElement('input');
				group_control.type = 'checkbox';
				li.appendChild(group_control);
				group_control.onclick = groupClickAll;
			}

			li.appendChild(document.createTextNode(group.getAttribute('title')));
			li.className = 'catalog-group';
			li.onclick = toggleGroup;
			li = null;
			if(parseBoolean(group.getAttribute('expand'))) {
				ul.className += ' catalog-group-expanded';
			} else {
				ul.className += ' catalog-group-collapsed';
			}
		} else {
			ul.className += ' catalog-group-expanded';
		}
		ul.id = getCatalogElementId();
		p.appendChild(ul);


		var args = OpenLayers.Util.getParameters();
		var overrideStatusList = {};
		if(args.on) {
			var on_list = new String(args.on).split(';');
			for(var i = 0; i < on_list.length; i++) {
				overrideStatusList[on_list[i]] = 'on';
			}
		}
		if(args.off) {
			var off_list = new String(args.off).split(';');
			for(var i = 0; i < off_list.length; i++) {
				overrideStatusList[off_list[i]] = 'off';
			}
		}
		for(var i = 0; i < group.childNodes.length; i++) {
			var tagName = group.childNodes[i].tagName;
			if(tagName) {
				if(tagName == 'group') {
					renderGroup(ul.id, group.childNodes[i]);
				} else if(tagName == 'layer') {
					renderLayer(ul.id, group.childNodes[i], overrideStatusList);
				}
			}
		}

		ul = null;
		p = null;
	}

	/* local variable of known paths */
	var known_paths = {};

	function iterateThroughLayers(pathString, fn) {
		var services = MyCatalog.getElementsByTagName('map-source');
		var paths = new String(pathString).split(':');
		/* n_XXX drastically improves loop iteration speed */
		var n_paths = paths.length;
		var n_services = services.length;

		/* known_paths is used to check for a previously known path. */
		var unknown_paths = [];
		for(var  p = 0; p < n_paths; p++) {
			if(known_paths[paths[p]]) {
				/* oh, we know about it, no need to do fancy stuff */
				fn(known_paths[paths[p]], paths[p]);
			} else {
				/* add it to the stack ... */
				unknown_paths.push(paths[p]);
			}
		}

		paths = unknown_paths;
		n_paths = paths.length;

		for(var i = 0; i < n_services && n_paths > 0; i++) {
			var root = services[i].getAttribute('name');
			var layers = services[i].getElementsByTagName('layer');
			var n_layers = layers.length;
			for(var l = 0; l < n_layers; l++) {
				var layerName = layers[l].getAttribute('name')
				for(var p = 0; p < n_paths; p++) {
					if(root+'/'+layerName == paths[p]) {
						var mapLayer = Map.getLayersByName(root)[0];
						if(mapLayer) {
							/* log the known path */
							known_paths[paths[p]] = mapLayer;
							/* run the function */
							fn(mapLayer, paths[p]);
						}
					}
				}

			}
		}
	
	}

	this.refreshLayer = function(pathString) {
		iterateThroughLayers(pathString, function(mapLayer) {
			/* This forces the image to reload */
			mapLayer.params['GEOMOOSE_REFRESH'] = (new Date()).getTime();
			mapLayer.redraw();
		});
	}

	this.fadeLayer = function(pathString) {
		iterateThroughLayers(pathString, function(mapLayer) {
			if(mapLayer.opacity == null) { mapLayer.opacity = 1; }
			var newOpac = mapLayer.opacity - CONFIGURATION.layer_controls.fade.change_percent/100;
			if(newOpac < 0) { newOpac = 0; }
			mapLayer.setOpacity(newOpac);
		});
	}


	this.unfadeLayer = function(pathString) {
		iterateThroughLayers(pathString, function(mapLayer) {
			if(mapLayer.opacity == null) { mapLayer.opacity = 1; }
			var newOpac = mapLayer.opacity + CONFIGURATION.layer_controls.unfade.change_percent/100;
			if(newOpac > 1) { newOpac = 1; }
			mapLayer.setOpacity(newOpac);
		});
	}

	this.upLayer = function(pathString) {
		iterateThroughLayers(pathString, function(mapLayer) {
			mapLayer.map.raiseLayer(mapLayer, 1);
		});
	}

	this.downLayer = function(pathString) {
		iterateThroughLayers(pathString, function(mapLayer) {
			mapLayer.map.raiseLayer(mapLayer, -1);
		});
	}

	this.cycleLayer = function(pathString) {
	}


	var legendImages = new Array();

	function updateLegendURLs(pathString) {
		iterateThroughLayers(pathString, function(mapLayer,layerPath) {
			var legendURL = false;
			if(!legendImages[layerPath]) {
				return false;	// nothing to do, no image available
			}
			var lname = layerPath.split('/').pop();
			var layers_array = [];
			if(mapLayer instanceof OpenLayers.Layer.MapServer) {
				legendURL = mapLayer.getURL(Map.getExtent()).replace('mode=map','mode=legend');

				/* specify THIS layer */
				legendURL = legendURL.replace(/layers=[\+\_\%a-zA-Z0-9\/]*\&/, 'layers='+lname+'&');
				if(mapLayer.params['layers']) {
					layers_array = mapLayer.params['layers'].split(' ');
				}
			}
			if(mapLayer instanceof OpenLayers.Layer.WMS || mapLayer instanceof OpenLayers.Layer.geomooseWMS) {
				legendURL = mapLayer.getURL(Map.getExtent()).replace('REQUEST=GetMap', 'REQUEST=GetLegendGraphic');
				legendURL += '&LAYER=' + lname;
				legendURL += '&SCALE=' + Map.getScale();
				if(mapLayer.params['LAYERS']) {
					layers_array = mapLayer.params['LAYERS'].split(',');
				}
			}
			/* let's make sure the layer is actually IN the map */
			var in_it = false;
			for(var i = 0, len = layers_array.length; i < len; i++) {
				if(layers_array[i] == lname) {
					in_it = true;
				}
			}

			if(!legendURL || !in_it) {
				legendURL = 'images/blank.gif';
			}
			document.getElementById(legendImages[layerPath]).setAttribute('src',legendURL);
		});
	}

	this.legendLayer = function(pathString) {
		var id = this.id;
		var parentControl = this;
		var legendParent = document.getElementById(id+'-legend');

		if(legendParent.className.indexOf('Visible') >= 0) {
			legendParent.className = legendParent.className.replace('Visible','Hidden');
			while(legendParent.childNodes[0]) {
				legendParent.removeChild(legendParent.childNodes[0]);
			}
			iterateThroughLayers(pathString, function(mapLayer, layerPath) {
				legendImages[layerPath] = false;
			});
			parentControl.className = parentControl.className.replace('sprite-control-legend-selected', '');
		} else {
			legendParent.className = legendParent.className.replace('Hidden','Visible');
			if(parentControl.className.indexOf('sprite-control-legend-selected') < 0) {
				parentControl.className += ' sprite-control-legend-selected'; 
			}
			iterateThroughLayers(pathString, function(mapLayer, layerPath) {
				if(!legendImages[layerPath]) {
					var legendImg = document.createElement('img');
					legendImg.className = 'catalog-legend-image';
					legendImg.id = 'catalog-legend-'+getCatalogElementId();
					legendImages[layerPath] = legendImg.id;
					legendParent.appendChild(legendImg);
					legendImg.setAttribute('src','images/blank.gif'); /* Default to prevent display issues */
					legendImg = null;
				}
			});
			updateLegendURLs(pathString);
		}
	}


	this.setLayerVisibility = function(pathString, visibility, default_on) {
		var services = MyCatalog.getElementsByTagName('map-source');
		var paths = new String(pathString).split(':');

		var foundSomething = false;

		for(var i = 0; i < services.length; i++) {
			var root = services[i].getAttribute('name');
			var layers = services[i].getElementsByTagName('layer');
			var updateRequired = false;
			for(var l = 0; l < layers.length; l++) {
				var layerName = layers[l].getAttribute('name')
				for(var p = 0; p < paths.length; p++) {
					if(root+'/'+layerName == paths[p]) {
						foundSomething = true;
						var v = 'false';
						if(visibility) {
							v = 'true';
						}
						if(default_on) {
							layers[l].setAttribute('default_on', v);
						}

						if(parseBoolean(visibility) != parseBoolean(layers[l].getAttribute('visible'), false)) {
							layers[l].setAttribute('visible', v);
							updateRequired = true;
						}
					}
				}
			}

			if(updateRequired) {
				var layersList = new Array();
				for(var l = 0; l < layers.length; l++) {
					if(parseBoolean(layers[l].getAttribute('visible'))) {
						layersList.push(layers[l].getAttribute('name'));
					}
				}
			
				var mapLayer = Map.getLayersByName(root)[0];
				var referenceLayer = [];
				
				if(CONFIGURATION.reference_map.enabled) {
					referenceLayer = ReferenceMap.ovmap.getLayersByName(root);
				}

				var updateLayers = [mapLayer];

				if(referenceLayer.length > 0) {
					referenceLayer = referenceLayer[0];
					updateLayers.push(referenceLayer);
				}


				for(var l = 0; l < updateLayers.length; l++) {
					var layer = updateLayers[l];
					if(layer instanceof OpenLayers.Layer.MapServer) {
						layer.params['layers'] = layersList.join(' ');
					} else if (layer instanceof OpenLayers.Layer.WMS || layer instanceof OpenLayers.Layer.geomooseWMS) {
						layer.params['LAYERS'] = layersList.join(',');
					}
					if(layersList.length == 0) {
						layer.setVisibility(false);
					} else {
						layer.setVisibility(true);
					}
					if((layer instanceof OpenLayers.Layer.Google || layer instanceof OpenLayers.Layer.VirtualEarth || layer instanceof OpenLayers.Layer.Yahoo || layer instanceof OpenLayers.Layer.XYZ || layer instanceof OpenLayers.Layer.ArcGIS93Rest) && layersList.length > 0) {
						Map.setBaseLayer(layer);
					}
					if(MapbookLoaded) {
						layer.redraw();
					}
				}
			}
			updateLegendURLs(pathString);
		}
		return foundSomething;
	}

	function multiSelectLayer(event, default_on) {
		var p = this.parentNode.parentNode.previousSibling;
		if(p && p.getElementsByTagName) {
			var inputs = p.getElementsByTagName('input');
			if(inputs.length > 0) {
				inputs[0].checked = false;
			}
		}
		MySelf.setLayerVisibility(this.value, this.checked, default_on);
	}

	function singleSelectLayer(event, default_on) {
		var p = this.parentNode.parentNode;
		var inputs = p.getElementsByTagName('input');
		for(var i = 0; i < inputs.length; i++) {
			if(inputs[i].name == this.name) {
				inputs[i].checked = false;
			}
			MySelf.setLayerVisibility(inputs[i].value, false, default_on);
		}
		this.checked = true;
		MySelf.setLayerVisibility(this.value, true, default_on);
	}

	function inScale(minscale, maxscale) {
		var scale = olMap.getScale();
		if(!maxscale && !minscale) { return true; }
		if(!maxscale && scale >= minscale) { return true; }
		if(!minscale && scale <= maxscale) { return true; }
		return (minscale <= scale && scale <= maxscale);
	}

	function showMetadata() {
		window.open(this.value);
	}

	function toggleControls() {
		var unique_id = this.id.split('-').pop();
		var controls_container = document.getElementById('layer-controls-container-catalog-'+unique_id);
		if(controls_container.className.indexOf(' Hidden') >= 0) {
			controls_container.className = controls_container.className.replace('Hidden', 'Visible');
		} else {
			controls_container.className = controls_container.className.replace(' Visible','');
			controls_container.className += ' Hidden';
		}
	}

	function renderLayer(groupElementId, layer, overrideStatusList) {
		var p = document.getElementById(groupElementId);
		var li = document.createElement('li');
		li.className = 'catalog-layer';
		p.appendChild(li);
		var cbox = document.createElement('input');
		cbox.id = 'catalog-'+layer.getAttribute('src');

		if(parseBoolean(layer.parentNode.getAttribute('multiple'),true)) {
			cbox.name = layer.parentNode.getAttribute('name');
			cbox.type = 'checkbox';
			cbox.value = layer.getAttribute('src');
			cbox.onclick = multiSelectLayer;
		} else {
			cbox.type = 'radio';
			cbox.value = layer.getAttribute('src');
			cbox.onclick = singleSelectLayer;
			if(overrideStatusList[cbox.value] == 'on') {
				var layers = layer.parentNode.getElementsByTagName('layer');
				for(var i = 0; i < layers.length; i++) {
					layers[i].setAttribute('status', 'off');
				}
			}
		}
		li.appendChild(cbox);

		var controlsId = getCatalogElementId();
		var layerTitle = document.createElement('span');
		layerTitle.id = 'layer-title-'+controlsId;
		layerTitle.className = 'catalog-layer-title';
		li.appendChild(layerTitle);

		layerTitle.appendChild(document.createTextNode(layer.getAttribute('title')));

		if(layer.getAttribute('tip')) {
			layerTitle.title = layer.getAttribute('tip');
		}

		if(layer.getAttribute('minscale')) {
			layerTitle.minscale = parseFloat(layer.getAttribute('minscale'));
		}

		if(layer.getAttribute('maxscale')) {
			layerTitle.maxscale = parseFloat(layer.getAttribute('maxscale'));
		}

		if(inScale(layerTitle.minscale, layerTitle.maxscale)) {
			layerTitle.className += ' catalog-inscale';
		} else {
			layerTitle.className += ' catalog-outscale';
		}


		
		var controlSpan = document.createElement('span');
		controlSpan.className = 'catalog-controls-container';
		controlSpan.id = 'layer-controls-container-'+controlsId;
		if(CONFIGURATION.catalog.toggle_controls) {
			layerTitle.onclick = toggleControls;
			layerTitle.className += ' ToggleControls';
		}
		if(!CONFIGURATION.catalog.show_controls) {
			controlSpan.className += ' Hidden';
		}
		li.appendChild(controlSpan);

		var legend = document.createElement('div');
		legend.className = 'catalog-legend Hidden';
		legend.id = controlsId+'-legend-legend';
		// Leave the legend "blank" for the time being.
		//legend.src = 'images/blank.gif';
		li.appendChild(legend);

		for(var control_name in CONFIGURATION.layer_controls) {
			var layerSwitch = CONFIGURATION.layer_controls[control_name].on;
			if(layer.getAttribute(control_name)) {
				layerSwitch = parseBoolean(layer.getAttribute(control_name));
			}
			var hideControl = false;
			if(control_name == 'legend' && !layerSwitch && parseBoolean(layer.getAttribute('show-legend'))) {
				layerSwitch = true;
				hideControl = true;
			}
			if(layerSwitch) {
				var toolDiv = document.createElement('div');
				toolDiv.className = 'sprite-control sprite-control-'+control_name;
				if(hideControl) { toolDiv.style.display = 'none'; }
				toolDiv.value = layer.getAttribute('src');
				toolDiv.controlFn = eval("MySelf."+control_name+"Layer");
				toolDiv.id = controlsId+'-'+control_name;
				toolDiv.onclick = function () {
					this.controlFn(this.value);
				}

				var tooltip = CONFIGURATION.layer_controls[control_name].tip;
				if(tooltip) {
					tooltip = tooltip.replace('%LAYER%', layer.getAttribute('title'));
					toolDiv.title = tooltip;
				}
				controlSpan.appendChild(toolDiv);

				if(control_name == 'legend' && parseBoolean(layer.getAttribute('show-legend'))) {
					toolDiv.onclick();
				}

				if(control_name == 'metadata') {
					toolDiv.onclick = showMetadata;
					var metadataURL = OpenLayers.Util.getXmlNodeValue(layer.getElementsByTagName('metadata')[0]);
					toolDiv.value = metadataURL;
				}
			}
		}
		/* check to see if the layer is "on" */
		var src = layer.getAttribute('src');
		var layer_status = parseBoolean(layer.getAttribute('status'));

		if(overrideStatusList[src] == 'on' || (layer_status && !(overrideStatusList[src] == 'off'))) {
			cbox.checked = true;
			/* tell the checkbox to log this as a "default" on layer */
			cbox.onclick(null, layer_status); 
		}

		cbox = null;
		li = null;
		p = null;
	}

	function onRefreshMap () {
		var p = document.getElementById(parentDivId);
		var span = p.getElementsByTagName('span');
		for(var i = 0; i < span.length; i++) {
			if(span[i].className.indexOf('catalog-layer-title') >= 0) {
				var layerTitle = span[i];
				if(inScale(layerTitle.minscale, layerTitle.maxscale)) {
					layerTitle.className = layerTitle.className.replace('catalog-outscale', 'catalog-inscale');
				} else {
					layerTitle.className = layerTitle.className.replace('catalog-inscale', 'catalog-outscale');
				}
				layerTitle = null;
			}
		}

		for(var pathString in legendImages) {
			updateLegendURLs(pathString);
		}
	}

	function toggleTool(obj, tool) {
		var visClassName = 'sprite-control-'+tool+'-selected';

		/* so it's current visible ... so remove it */
		if(obj.className.indexOf(visClassName) >= 0) {
			obj.className = obj.className.replace(' '+visClassName, '');
			obj.className = obj.className.replace(visClassName, '');
		} else {
			obj.className = obj.className + ' ' + visClassName;
			onSelectTool.push(function() {
				toggleTool(obj, tool);
			});
		}

	}

	this.onLoadedMapbook = function(mapbookXML) {
		MyCatalog = mapbookXML;
		var catalog = mapbookXML.getElementsByTagName('catalog')[0];
		var layerControls = mapbookXML.getElementsByTagName('layer-control');
		for(var i = 0; i < layerControls.length; i++) {
			CONFIGURATION.layer_controls[layerControls[i].getAttribute('name')] = {'on' : parseBoolean(layerControls[i].getAttribute('on')), 'tip' : layerControls[i].getAttribute('tip')}	
			var funcString = "function(paths) { selectTool(this); toggleTool(this, '%service_name%'); GeoMOOSE.startService('%service_name%', {'%input%' : paths}); }";
			funcString = funcString.replace(/%service_name%/g, layerControls[i].getAttribute('service-name'));
			funcString = funcString.replace('%input%', layerControls[i].getAttribute('layer-input'));


			eval("MySelf."+layerControls[i].getAttribute('name')+"Layer = "+funcString);
		}

		renderGroup(parentDivId, catalog);

		olMap.events.register('moveend', this, onRefreshMap);
		onRefreshMap();
		MapbookLoaded = true;
	}

	/* return an array of the visible layers */
	this.getVisibleLayers = function() {
		var visible_layers = [];
		var sources = MyCatalog.getElementsByTagName('map-source');
		for(var i = 0; i < sources.length; i++) {
			var layers = sources[i].getElementsByTagName('layer');
			var name = sources[i].getAttribute('name')+'/';
			for(var j = 0; j < layers.length; j++) {
				if(parseBoolean(layers[j].getAttribute('visible'))) {
					visible_layers.push(name+layers[j].getAttribute('name'));
				}
			}
		}
		return visible_layers;
	}

	this.getLayerDifferences = function () {
		var layers_diff = {on: [], off: []};
		var sources = MyCatalog.getElementsByTagName('map-source');
		for(var i = 0; i < sources.length; i++) {
			var layers = sources[i].getElementsByTagName('layer');
			var name = sources[i].getAttribute('name')+'/';
			for(var j = 0; j < layers.length; j++) {
				var layer_name = name+layers[j].getAttribute('name');
				var is_vis_now = parseBoolean(layers[j].getAttribute('visible'));
				var was_vis_default = parseBoolean(layers[j].getAttribute('default_on'));
				if(is_vis_now != was_vis_default) {
					if(is_vis_now) {
						layers_diff.on.push(layer_name);
					} else {
						layers_diff.off.push(layer_name);
					}
				}
			}
		}
		return layers_diff;
	
	}

	function getSourcesByAttribute(sourceAttribute, sourceValue) {
		var sources = MyCatalog.getElementsByTagName('map-source');
		var return_sources = [];
		for(var i = 0; i < sources.length; i++) {
			if(sources[i].getAttribute(sourceAttribute) == sourceValue) {
				return_sources.push(sources[i]);
			}
		}
		return return_sources;
	}

	function getSourcesByName(sourceName) {
		return getSourcesByAttribute('name', sourceName);
	}

	function setNodeValue(node, value) {
		while(node.childNodes && node.childNodes[0]) { node.removeChild(node.childNodes[0]); }
		node.appendChild(MyCatalog.createTextNode(value));
	}

	/* set the layer url */
	this.setLayerUrl = function(layerName, layerUrl) {
		var source = getSourcesByName(layerName)[0];
		var url = source.getElementsByTagName('url')[0];

		if(!url) {
			url = MyCatalog.createElement('url');
			source.appendChild(url);
		}
		setNodeValue(url, layerUrl);

		var layer = Map.getLayersByName(layerName)[0];
		layer.url = layerUrl;
	}

	/* get the layer url */
	this.getLayerUrl = function(layerName) {
		return Map.getLayersByName(layerName)[0].url;
	}

	/* get the layer opacity */
	this.getLayerOpacity = function(layerName) {
		var lay = Map.getLayersByName(layerName)[0];
		if(lay.opacity != null) {
			return lay.opacity;
		}
		return 1;
	}


	/* update layer params */
	this.setLayerParameters = function(layerName, parameters) {
		var source = getSourcesByName(layerName)[0];
		var params = source.getElementsByTagName('param');

		var layer = Map.getLayersByName(layerName)[0];
		for(var p in parameters) {
			layer.params[p] = parameters[p];
		}

		for(var p = 0; p < params.length; p++) {
			var name = params[p].getAttribute('name');
			if(parameters[name]) {
				setNodeValue(params[p], parameters[name]);
				delete parameters[name];
			}
		}

		for(var n in parameters) {
			var param = MyCatalog.createElement('param');
			param.setAttribute('name', n);
			source.appendChild(param);
			/*setNodeValue(param, parameters[n]);*/
			param.setAttribute('value', parameters[n]);
		}

	}

	/* remove a layer parameter */
	this.removeLayerParameter = function(layerName, parameterName) {
		var source = getSourcesByName(layerName)[0];
		var params = source.getElementsByTagName('param');
		var paramToDelete = false;
		for(var p = 0; p < params.length; p++) {
			var name = params[p].getAttribute('name');
			if(name == parameterName) { paramToDelete = params[p]; }
		}
		if(paramToDelete) {
			source.removeChild(paramToDelete);
		}

		var layer = Map.getLayersByName(layerName)[0];
		if(layer.params[parameterName]) {
			delete layer.params[parameterName];
		}
	}
	/* get the layer parameters as an object */
	this.getLayerParameters = function(layerName) {
		var source = getSourcesByName(layerName)[0];
		var params = source.getElementsByTagName('param');
		var parameters = {};
		for(var p = 0; p < params.length; p++) {
			parameters[params[p].getAttribute('name')] = params[p].getAttribute('value');
		}
		return parameters;
	}


	this.getLayerStatus = function() {
		var catalog = MyCatalog.getElementsByTagName('catalog');
		var layers = catalog.getElementsByTagName('layer');

		for(var i = 0; i < layers.length; i++) {
		}

		
	}
}
