/*
Copyright (c) 2009, Dan "Ducky" Little & GeoMOOSE.org

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/


/* converts from sq inches to another unit */
var AREA_CONVERSION = {
	'mi' : 1/.000000000249097669,
	'acre' : 1/.000000159422508,
	'yd' : 1/0.000771604938,
	'ft' : 1/0.00694444444,
	'm' : 1/0.00064516,
	'in' : 1
};


function startupMeasureLength(event) {
	startupMeasureTool('length');
}

function startupMeasureArea(event) {
	startupMeasureTool('area');
}

function startupMeasureTool(mType) {
	Tabs.showTab(CONFIGURATION.show_results_in);

	var trans = {'length' : 'Length', 'area' : 'Area' };

	var p = document.getElementById(Tabs.getTab(CONFIGURATION.show_results_in));

	p.innerHTML = '';

	var title = document.createElement('div');
	p.appendChild(title);
	title.appendChild(document.createTextNode('Measure '+trans[mType]));

	p.appendChild(document.createElement('br'));

	var infoTable = document.createElement('table');
	var infoTbody = document.createElement('tbody');
	p.appendChild(infoTable);
	infoTable.appendChild(infoTbody);

	var totalRow = document.createElement('tr');
	var totalTitle = document.createElement('th');
	var totalCell = document.createElement('td');

	infoTbody.appendChild(totalRow);
	totalRow.appendChild(totalTitle);
	totalRow.appendChild(totalCell);

	var segmentRow = document.createElement('tr');
	var segmentTitle = document.createElement('th');
	var segmentCell = document.createElement('td');

	infoTbody.appendChild(segmentRow);
	segmentRow.appendChild(segmentTitle);
	segmentRow.appendChild(segmentCell);

	if(CONFIGURATION.measure_tool.show_heading) {
		var heading_row = document.createElement('tr');
		var heading_title = document.createElement('th');
		var heading_cell = document.createElement('td');
		infoTbody.appendChild(heading_row);
		heading_row.appendChild(heading_title);
		heading_row.appendChild(heading_cell);

		heading_title.appendChild(document.createTextNode('Heading:'));

		var heading_input = document.createElement('input');
		heading_input.id = 'measuretool-input-heading';
		heading_input.size = 14;
		heading_cell.appendChild(heading_input);
	}
		

	var unitsRow = document.createElement('tr');
	var unitsTitle = document.createElement('th');
	var unitsCell = document.createElement('td');

	infoTbody.appendChild(unitsRow);
	unitsRow.appendChild(unitsTitle);
	unitsRow.appendChild(unitsCell);

	totalTitle.appendChild(document.createTextNode('Total '+trans[mType]+':'));
	
	var totalInput = document.createElement('input');
	totalInput.size = 14;
	totalInput.id = 'measuretool-input-total';
	totalCell.appendChild(totalInput);


	segmentTitle.appendChild(document.createTextNode('Segment '+trans[mType]+':'));
	var segmentInput = document.createElement('input');
	segmentInput.size = 14;
	segmentInput.id = 'measuretool-input-segment';
	segmentCell.appendChild(segmentInput);

	unitsTitle.appendChild(document.createTextNode('Units:'));

	var unitsDropdown = document.createElement('select');
	var units = {'Miles' : 'mi', 'Feet' : 'ft', 'Meters' : 'm'};

	unitsDropdown.id = 'measuretool-units';
	unitsCell.appendChild(unitsDropdown);

	if(mType == 'area') {
		units['Yards'] = 'yd';
		units['Acres'] = 'acre';
	}
	unitsDropdown.onchange = measureToolUpdateUnits;

	for(var u in units) {
		var opt = document.createElement('option');
		opt.value = units[u];
		unitsDropdown.appendChild(opt);
		opt.appendChild(document.createTextNode(u));
		if((mType != 'area' && units[u] == CONFIGURATION.measure_tool.line_units)
			|| (mType == 'area' && units[u] == CONFIGURATION.measure_tool.area_units)) {
			opt.selected = true;
		}
	}

	var clearRow = document.createElement('tr');
	var blankCell = document.createElement('td');
	var clearCell = document.createElement('td');

	infoTbody.appendChild(clearRow);
	clearRow.appendChild(blankCell);
	clearRow.appendChild(clearCell);

	var clearButton = document.createElement('input');
	clearButton.type = 'button';
	clearButton.value = 'Clear';
	clearButton.onclick = measureToolClear;
	clearCell.appendChild(clearButton);

	var measureTable = document.createElement('table');
	var measureTbody = document.createElement('tbody');
	measureTbody.id = 'measuretool-tbody';

	p.appendChild(measureTable);
	measureTable.appendChild(measureTbody);
}

function measureToolClear(event) {
	var measureTable = document.getElementById('measuretool-tbody');
	while(measureTable.childNodes[0]) {
		measureTable.removeChild(measureTable.childNodes[0]);
	}

	var measureTools = ['measure','measurearea'];
	for(var m = 0; m < measureTools.length; m++) {
		var tool = measureTools[m];
		Tools[tool].measure(new OpenLayers.Geometry.Point(), 'measure');
		if(Tools[tool].handler.layer) {
			Tools[tool].handler.layer.removeFeatures(Tools[tool].handler.layer.features);
		}
	}
}

function measureToolPartial(event) {
	var units = document.getElementById('measuretool-units');
	units = units.getElementsByTagName('option')[units.selectedIndex].value;

	var total = document.getElementById('measuretool-input-total');
	var segment = document.getElementById('measuretool-input-segment');

	var t = document.getElementById('measuretool-tbody');

	var active_tool = Tools['measure'];
	var conversion_list = OpenLayers.INCHES_PER_UNIT;
	if(Tools['measurearea'].active) {
		active_tool = Tools['measurearea'];
		conversion_list = AREA_CONVERSION;
	}

	/* get the sum of hte lengths */
	total.value = commifyNumber((active_tool.total / conversion_list[units]).toFixed(CONFIGURATION.measure_tool.precision));

	while(t.firstChild) {
		t.removeChild(t.firstChild);
	}

	for(var i = active_tool.segments.length - 1; i >= 0; i--) {
		var row = document.createElement('tr');
		t.appendChild(row);

		var segmentTitle = document.createElement('th');
		row.appendChild(segmentTitle);
		segmentTitle.appendChild(document.createTextNode((i+1)+':'));

		var segmentLength = document.createElement('td');
		row.appendChild(segmentLength);

		var segment_measure = active_tool.segments[i] / conversion_list[units];

		segmentLength.appendChild(document.createTextNode(commifyNumber(segment_measure.toFixed(CONFIGURATION.measure_tool.precision))));

		var segmentUnits = document.createElement('td');
		row.appendChild(segmentUnits);
		segmentUnits.appendChild(document.createTextNode(units));
	}
}

function measureToolLengthLog(event) {
	var units = document.getElementById('measuretool-units');
	units = units.getElementsByTagName('option')[units.selectedIndex].value;
	var segment = document.getElementById('measuretool-input-segment');
	var total = document.getElementById('measuretool-input-total');

	var active_tool = Tools['measure'];
	if(Tools['measurearea'].active) {
		active_tool = Tools['measurearea'];
	}

	total_measure = active_tool.total;
	segment_measure = event.measure;

	/* convert to the proper units */
	if(event.order == 1) {
		total_measure = total_measure / OpenLayers.INCHES_PER_UNIT[units];
		segment_measure = segment_measure / OpenLayers.INCHES_PER_UNIT[units];
	} else {
		total_measure = total_measure / AREA_CONVERSION[units];
		segment_measure = segment_measure / AREA_CONVERSION[units];
	}

	total.value = commifyNumber(total_measure.toFixed(CONFIGURATION.measure_tool.precision));
	segment.value = commifyNumber(segment_measure.toFixed(CONFIGURATION.measure_tool.precision));

	if(CONFIGURATION.measure_tool.show_heading) {
		var len = event.geometry.components.length;
		var bearing = '';
		if(len >= 2) {
			var bearing = 'N0-0-0E';
			var point_a = event.geometry.components[len-2];
			var point_b = event.geometry.components[len-1];

			var rise = point_b.y - point_a.y;
			var run = point_b.x - point_a.x;
			if(rise == 0) {
				if(point_a.x > point_b.x) {
					bearing = 'Due West';
				} else {
					bearing = 'Due East';
				}
			} else if(run == 0) {
				if(point_a.y > point_b.y) {
					bearing = 'Due South';
				} else {
					bearing = 'Due North';
				}
			} else {
				var ns_quad = 'N';
				var ew_quad = 'E';
				if(rise < 0) {
					ns_quad = 'S';
				}
				if(run < 0) {
					ew_quad = 'W';
				}
				/* we've determined the quadrant, so we can make these absolute */
				rise = Math.abs(rise);
				run = Math.abs(run);
				/* convert to degrees */
				var degrees = Math.atan(rise/run) / (2*Math.PI) * 360;
				/* and to DMS ... */
				var d = parseInt(degrees);
				var t = (degrees - d) * 60;
				var m = parseInt(t);
				var s = parseInt(60 * (t-m));

				bearing = ns_quad+d+'-'+m+'-'+s+ew_quad;

			}
			document.getElementById('measuretool-input-heading').value = bearing;

		}
	}
	//console.log(event.geometry);
}

function measureToolUpdateUnits() {
	var units = document.getElementById('measuretool-units');
	units = units.getElementsByTagName('option')[units.selectedIndex].value;
	var segment = document.getElementById('measuretool-input-segment');

	var active_tool = Tools['measure'];
	var conversion_list = OpenLayers.INCHES_PER_UNIT;
	if(Tools['measurearea'].active) {
		active_tool = Tools['measurearea'];
		conversion_list = AREA_CONVERSION;
	}

	segment.value = commifyNumber((active_tool.segments[active_tool.segments.length-1] / conversion_list[units]).toFixed(CONFIGURATION.measure_tool.precision));
	measureToolPartial();
}
